/******************************************************************************\

          This file is part of the C! library.  A.K.A the cbang library.

              Copyright (c) 2003-2012, Cauldron Development LLC
                 Copyright (c) 2003-2012, Stanford University
                             All rights reserved.

        The C! library is free software: you can redistribute it and/or
        modify it under the terms of the GNU Lesser General Public License
        as published by the Free Software Foundation, either version 2.1 of
        the License, or (at your option) any later version.

        The C! library is distributed in the hope that it will be useful,
        but WITHOUT ANY WARRANTY; without even the implied warranty of
        MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
        Lesser General Public License for more details.

        You should have received a copy of the GNU Lesser General Public
        License along with the C! library.  If not, see
        <http://www.gnu.org/licenses/>.

        In addition, BSD licensing may be granted on a case by case basis
        by written permission from at least one of the copyright holders.
        You may request written permission by emailing the authors.

                For information regarding this software email:
                               Joseph Coffland
                        joseph@cauldrondevelopment.com

\******************************************************************************/

#ifndef CBANG_HTTP_MESSAGE_H
#define CBANG_HTTP_MESSAGE_H

#include "Header.h"
#include "Cookie.h"

#include <cbang/SmartPointer.h>
#include <cbang/StdTypes.h>

#include <string>
#include <vector>
#include <iostream>


namespace cb {
  namespace HTTP {
    class Message : public Header {
    public:
      static const char *TIME_FORMAT;

    protected:
      float version;
      bool chunked;
      typedef std::map<std::string, SmartPointer<Cookie> > cookies_t;
      cookies_t cookies;

    public:
      Message(float version = 1.1) : version(version), chunked(false) {}
      virtual ~Message() {}

      void reset() {clear(); chunked = false; cookies.clear();}

      float getVersion() const {return version;}
      void setVersion(float version) {this->version = version;}

      bool isChunked() const {return chunked;}
      void setChunked(bool chunked) {this->chunked = chunked;}

      bool hasCookie(const std::string &name) const;
      const SmartPointer<Cookie> &getCookie(const std::string &name) const;
      void setCookie(const SmartPointer<Cookie> &cookie);

      void setPersistent(bool x);
      bool isPersistent() const;

      std::string getHeader() const;

      std::string getVersionString() const;
      void readVersionString(const std::string &s);

      void setSessionID(const std::string &id, const std::string &name = "SID");
      std::string getSessionID(const std::string &name = "SID") const;

      bool isRequest() const {return !isResponse();}

      virtual bool isResponse() const = 0;
      virtual std::string getHeaderLine() const = 0;
      virtual void readHeaderLine(const std::string &line) = 0;

      virtual void read(const std::string &data);
      virtual std::istream &read(std::istream &stream);
      virtual std::ostream &write(std::ostream &stream) const;
    };

    inline std::ostream &operator<<(std::ostream &stream, const Message &m) {
      return m.write(stream);
    }
  }
}

#endif // CBANG_HTTP_MESSAGE_H

