/******************************************************************************\

          This file is part of the C! library.  A.K.A the cbang library.

              Copyright (c) 2003-2012, Cauldron Development LLC
                 Copyright (c) 2003-2012, Stanford University
                             All rights reserved.

        The C! library is free software: you can redistribute it and/or
        modify it under the terms of the GNU Lesser General Public License
        as published by the Free Software Foundation, either version 2.1 of
        the License, or (at your option) any later version.

        The C! library is distributed in the hope that it will be useful,
        but WITHOUT ANY WARRANTY; without even the implied warranty of
        MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
        Lesser General Public License for more details.

        You should have received a copy of the GNU Lesser General Public
        License along with the C! library.  If not, see
        <http://www.gnu.org/licenses/>.

        In addition, BSD licensing may be granted on a case by case basis
        by written permission from at least one of the copyright holders.
        You may request written permission by emailing the authors.

                For information regarding this software email:
                               Joseph Coffland
                        joseph@cauldrondevelopment.com

\******************************************************************************/

#include "WebContext.h"

#include "WebHandler.h"
#include "Connection.h"

#include <cbang/net/URI.h>
#include <cbang/os/SystemUtilities.h>

using namespace std;
using namespace cb::HTTP;


WebContext::WebContext(WebHandler &handler, Connection &con) :
  Context(con), cb::Script::Environment("HTTP Context", &handler),
  URIEvaluator(con.getRequest().getURI(), *this), handler(handler),
  dynamic(false) {
}


bool WebContext::eval(const cb::Script::Context &ctx) {
  if (cb::Script::Environment::eval(ctx)) return true;

  // TODO other standard server vars
  //   See: http://php.net/manual/en/reserved.variables.server.php

  string func = ctx.args[0];

  if (func == "include") {
    if (ctx.args.size() != 2) ctx.args.invalidNum();

    string path = ctx.handler.eval(ctx.args[1]);

    // TODO move this path code to WebHandler
    if (!path.empty() && path[0] != '/') {
      string parent;
      if (paths.empty()) parent = con.getRequest().getURI().getPath();
      else parent = paths.back();

      // Find last /
      size_t pos = parent.find_last_of('/');
      if (pos != string::npos) path = parent.substr(0, pos + 1) + path;

      // Remove ..'s
      vector<string> parts;
      SystemUtilities::splitPath(path, parts);
      unsigned j = 0;
      for (unsigned i = 0; i < parts.size(); i++) {
        if (parts[i] == "..") {
          if (!j) THROWS("Invalid path: " << path);
          j--;

        } else if (parts[i] != ".") parts[j++] = parts[i];
      }

      parts.resize(j);
      path = SystemUtilities::joinPath(parts);
      if (!parent.empty() && parent[0] == '/') path = string("/") + path;
    }

    paths.push_back(path);
    try {
      handler.handlePage(*this, ctx.stream, path);
    } catch (...) {
      paths.pop_back();
      throw;
    }
    paths.pop_back();

  } else if (func == "REMOTE_ADDR") {
    ctx.stream << con.getClientIP();

  } else if (func == "REQUEST_METHOD") {
    ctx.stream << con.getRequest().getMethod().toString();

  } else if (func == "QUERY_STRING") {
    ctx.stream << con.getRequest().getURI().getQuery();

  } else if (func == "REQUEST_URI") {
    ctx.stream << con.getRequest().getURI().getPath();

  } else return false;

  return true;
}
