/******************************************************************************\

          This file is part of the C! library.  A.K.A the cbang library.

              Copyright (c) 2003-2012, Cauldron Development LLC
                 Copyright (c) 2003-2012, Stanford University
                             All rights reserved.

        The C! library is free software: you can redistribute it and/or
        modify it under the terms of the GNU Lesser General Public License
        as published by the Free Software Foundation, either version 2.1 of
        the License, or (at your option) any later version.

        The C! library is distributed in the hope that it will be useful,
        but WITHOUT ANY WARRANTY; without even the implied warranty of
        MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
        Lesser General Public License for more details.

        You should have received a copy of the GNU Lesser General Public
        License along with the C! library.  If not, see
        <http://www.gnu.org/licenses/>.

        In addition, BSD licensing may be granted on a case by case basis
        by written permission from at least one of the copyright holders.
        You may request written permission by emailing the authors.

                For information regarding this software email:
                               Joseph Coffland
                        joseph@cauldrondevelopment.com

\******************************************************************************/

#include "Base64.h"

#include <cbang/Exception.h>


using namespace std;
using namespace cb;


namespace cb {
  namespace Base64 {

    const char *encodeTable =
      "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";

    const char decodeTable[256] = {
      -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
      -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
      -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, 62, -1, -1, -1, 63,
      52, 53, 54, 55, 56, 57, 58, 59, 60, 61, -1, -1, -1,'=', -1, -1,
      -1,  0,  1,  2,  3,  4,  5,  6,  7,  8,  9, 10, 11, 12, 13, 14,
      15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, -1, -1, -1, -1, -1,
      -1, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 37, 38, 39, 40,
      41, 42, 43, 44, 45, 46, 47, 48, 49, 50, 51, -1, -1, -1, -1, -1,
      -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
      -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
      -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
      -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
      -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
      -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
      -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
      -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
    };


    string encode(const string &s) {
      return encode(s.c_str(), s.length());
    }


    string encode(const char *s, unsigned length) {
      const char *end = s + length;
      string result;
      result.reserve(length / 3 * 4 + 4);

      int pad = 0;
      char a, b, c;
      while (s != end) {
        a = *s++;
        if (s == end) {c = b = 0; pad = 2;}
        else {
          b = *s++;
          if (s == end) {c = 0; pad = 1;}
          else c = *s++;
        }

        result += encodeTable[63 & (a >> 2)];
        result += encodeTable[63 & (a << 4 | b >> 4)];
        if (pad == 2) result += '=';
        else result += encodeTable[63 & (b << 2 | c >> 6)];
        if (pad) result += '=';
        else result += encodeTable[63 & c];
      }

      return result;
    }


    string decode(const string &s) {
      string result;
      result.reserve(s.length() / 4 * 3 + 1);

      for (string::const_iterator it = s.begin(); it != s.end();) {
        char w = decodeTable[(int)*it++];
        char x = it == s.end() ? -1 : decodeTable[(int)*it++];
        char y = it == s.end() ? -1 : decodeTable[(int)*it++];
        char z = it == s.end() ? -1 : decodeTable[(int)*it++];

        if (w == -1 || x == -1 || y == -1 || z == -1)
          THROW("Invalid Base64 data");

        result += (char)(w << 2 | x >> 4);
        if (y != '=') {
          result += (char)(x << 4 | y >> 2);
          if (z != '=') result += (char)(y << 6 | z);
        }
      }

      return result;
    }
  }
}
