/******************************************************************************\

          This file is part of the C! library.  A.K.A the cbang library.

              Copyright (c) 2003-2012, Cauldron Development LLC
                 Copyright (c) 2003-2012, Stanford University
                             All rights reserved.

        The C! library is free software: you can redistribute it and/or
        modify it under the terms of the GNU Lesser General Public License
        as published by the Free Software Foundation, either version 2.1 of
        the License, or (at your option) any later version.

        The C! library is distributed in the hope that it will be useful,
        but WITHOUT ANY WARRANTY; without even the implied warranty of
        MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
        Lesser General Public License for more details.

        You should have received a copy of the GNU Lesser General Public
        License along with the C! library.  If not, see
        <http://www.gnu.org/licenses/>.

        In addition, BSD licensing may be granted on a case by case basis
        by written permission from at least one of the copyright holders.
        You may request written permission by emailing the authors.

                For information regarding this software email:
                               Joseph Coffland
                        joseph@cauldrondevelopment.com

\******************************************************************************/

#include "List.h"

#include "None.h"
#include "Boolean.h"
#include "Number.h"
#include "String.h"

#include <cbang/Exception.h>
#include <cbang/String.h>

#include <cctype>

using namespace std;
using namespace cb;
using namespace cb::PyON;


void List::appendNone() {
  append(new None);
}


void List::appendBoolean(bool value) {
  append(new Boolean(value));
}


void List::append(double value) {
  append(new Number(value));
}


void List::append(const std::string &value) {
  append(new String(value));
}


void List::write(const Writer &writer) const {
  writer.stream << "[";
  Writer next = writer.next();

  for (const_iterator it = begin(); it != end(); it++) {
    if (it != begin()) {
      writer.stream << ',';
      if (compact) writer.stream << ' ';
    }
    if (!compact) {
      writer.stream << '\n';
      next.indent();
    }
    (*it)->write(next);
  }

  if (!compact && !empty()) {
    writer.stream << '\n';
    writer.indent();
  }
  writer.stream << "]";
}


void List::read(Reader &reader) {
  reader.match("[");

  while (reader.good()) {
    char c = reader.next();

    if (c == ']') {
      reader.advance();
      return;

    } else {
      push_back(reader.parse());

      c = reader.next();
      if (c == ',') reader.advance();
      else if (reader.match(",]") == ']') return;
    }
  }
}
