/******************************************************************************\

          This file is part of the C! library.  A.K.A the cbang library.

              Copyright (c) 2003-2012, Cauldron Development LLC
                 Copyright (c) 2003-2012, Stanford University
                             All rights reserved.

        The C! library is free software: you can redistribute it and/or
        modify it under the terms of the GNU Lesser General Public License
        as published by the Free Software Foundation, either version 2.1 of
        the License, or (at your option) any later version.

        The C! library is distributed in the hope that it will be useful,
        but WITHOUT ANY WARRANTY; without even the implied warranty of
        MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
        Lesser General Public License for more details.

        You should have received a copy of the GNU Lesser General Public
        License along with the C! library.  If not, see
        <http://www.gnu.org/licenses/>.

        In addition, BSD licensing may be granted on a case by case basis
        by written permission from at least one of the copyright holders.
        You may request written permission by emailing the authors.

                For information regarding this software email:
                               Joseph Coffland
                        joseph@cauldrondevelopment.com

\******************************************************************************/

#ifndef CBANG_PYON_READER_H
#define CBANG_PYON_READER_H

#include <cbang/SmartPointer.h>

#include <istream>

namespace cb {
  namespace PyON {
    class Value;

    class Reader {
      unsigned line;
      unsigned column;
      std::istream &stream;

    public:
      Reader(std::istream &stream) : line(0), column(0), stream(stream) {}
      SmartPointer<Value> parse();

      unsigned getLine() const {return line;}
      unsigned getColumn() const {return column;}

      char get();
      char next();
      void advance() {stream.get();}
      char match(const char *chars);
      bool good() const {return stream.good();}

      const std::string parseKeyword();
      bool parseBoolean();
      double parseNumber();
      const std::string parseString();
    };
  }
}

#endif // CBANG_PYON_READER_H

