/******************************************************************************\

          This file is part of the C! library.  A.K.A the cbang library.

              Copyright (c) 2003-2012, Cauldron Development LLC
                 Copyright (c) 2003-2012, Stanford University
                             All rights reserved.

        The C! library is free software: you can redistribute it and/or
        modify it under the terms of the GNU Lesser General Public License
        as published by the Free Software Foundation, either version 2.1 of
        the License, or (at your option) any later version.

        The C! library is distributed in the hope that it will be useful,
        but WITHOUT ANY WARRANTY; without even the implied warranty of
        MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
        Lesser General Public License for more details.

        You should have received a copy of the GNU Lesser General Public
        License along with the C! library.  If not, see
        <http://www.gnu.org/licenses/>.

        In addition, BSD licensing may be granted on a case by case basis
        by written permission from at least one of the copyright holders.
        You may request written permission by emailing the authors.

                For information regarding this software email:
                               Joseph Coffland
                        joseph@cauldrondevelopment.com

\******************************************************************************/

#ifndef CBANG_PYON_VALUE_H
#define CBANG_PYON_VALUE_H

#include <cbang/SmartPointer.h>
#include <cbang/Exception.h>

#include "Writer.h"
#include "Reader.h"

#include <istream>

namespace cb {
  namespace PyON {
    class List;
    class Dict;

    class Value {
    public:
      typedef enum {
        PYON_NONE,
        PYON_BOOLEAN,
        PYON_NUMBER,
        PYON_STRING,
        PYON_LIST,
        PYON_DICT,
      } type_t;

      virtual ~Value() {}

      virtual type_t getType() const = 0;

      virtual bool getBoolean() const {CBANG_THROW("Value is not a boolean");}
      virtual double getNumber() const {CBANG_THROW("Value is not a number");}
      virtual std::string &getString() {CBANG_THROW("Value is not a string");}
      virtual const std::string &getString() const
      {CBANG_THROW("Value is not a string");}
      virtual List &getList() {CBANG_THROW("Value is not a list");}
      virtual const List &getList() const {CBANG_THROW("Value is not a list");}
      virtual Dict &getDict() {CBANG_THROW("Value is not a dictionary");}
      virtual const Dict &getDict() const
      {CBANG_THROW("Value is not a dictionary");}

      virtual void write(const Writer &writer) const = 0;
      virtual void read(Reader &reader) = 0;
    };

    static inline
    std::ostream &operator<<(std::ostream &stream, const Value &value) {
      value.write(Writer(stream)); return stream;
    }

    static inline
    std::istream &operator>>(std::istream &stream, Value &value) {
      Reader reader(stream);
      value.read(reader); return stream;
    }

    typedef SmartPointer<Value> ValuePtr;
  }
}

#endif // CBANG_PYON_VALUE_H

