/******************************************************************************\

          This file is part of the C! library.  A.K.A the cbang library.

              Copyright (c) 2003-2012, Cauldron Development LLC
                 Copyright (c) 2003-2012, Stanford University
                             All rights reserved.

        The C! library is free software: you can redistribute it and/or
        modify it under the terms of the GNU Lesser General Public License
        as published by the Free Software Foundation, either version 2.1 of
        the License, or (at your option) any later version.

        The C! library is distributed in the hope that it will be useful,
        but WITHOUT ANY WARRANTY; without even the implied warranty of
        MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
        Lesser General Public License for more details.

        You should have received a copy of the GNU Lesser General Public
        License along with the C! library.  If not, see
        <http://www.gnu.org/licenses/>.

        In addition, BSD licensing may be granted on a case by case basis
        by written permission from at least one of the copyright holders.
        You may request written permission by emailing the authors.

                For information regarding this software email:
                               Joseph Coffland
                        joseph@cauldrondevelopment.com

\******************************************************************************/

#include "BlowfishCipher.h"

#include <cbang/Exception.h>

#include <openssl/evp.h>

using namespace cb;

static const unsigned char defaultIV[] = {0, 0, 0, 0, 0, 0, 0, 0};


BlowfishCipher::BlowfishCipher(bool encrypt, const unsigned char key[16],
                               const unsigned char iv[8], blowfish_t type) :
  Cipher(getCyperType(type), encrypt, key, iv ? iv : defaultIV) {}


const EVP_CIPHER *BlowfishCipher::getCyperType(blowfish_t type) {
  switch (type) {
  case BF_ECB: return EVP_bf_ecb(); break;
  case BF_CBC: return EVP_bf_cbc(); break;
  case BF_CFB64: return EVP_bf_cfb64(); break;
  case BF_OFB: return EVP_bf_ofb(); break;
  default: THROWS("Unknown blowfish type " << (int)type);
  }
}
