/******************************************************************************\

          This file is part of the C! library.  A.K.A the cbang library.

              Copyright (c) 2003-2012, Cauldron Development LLC
                 Copyright (c) 2003-2012, Stanford University
                             All rights reserved.

        The C! library is free software: you can redistribute it and/or
        modify it under the terms of the GNU Lesser General Public License
        as published by the Free Software Foundation, either version 2.1 of
        the License, or (at your option) any later version.

        The C! library is distributed in the hope that it will be useful,
        but WITHOUT ANY WARRANTY; without even the implied warranty of
        MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
        Lesser General Public License for more details.

        You should have received a copy of the GNU Lesser General Public
        License along with the C! library.  If not, see
        <http://www.gnu.org/licenses/>.

        In addition, BSD licensing may be granted on a case by case basis
        by written permission from at least one of the copyright holders.
        You may request written permission by emailing the authors.

                For information regarding this software email:
                               Joseph Coffland
                        joseph@cauldrondevelopment.com

\******************************************************************************/

#include "MD5.h"

#include <cbang/Exception.h>
#include <cbang/Zap.h>
#include <cbang/String.h>

#include <openssl/md5.h>

using namespace std;
using namespace cb;

namespace cb {
  struct MD5::private_t  {
    MD5_CTX ctx;
  };


  MD5::MD5(const std::string &s) : p(new private_t), finalized(false) {
    reset();
    add(s);
  }


  MD5::~MD5() {
    zap(p);
  }


  void MD5::add(const string &s) {
    add(s.c_str(), s.length());
  }


  void MD5::add(const char *s, unsigned n) {
    if (finalized) THROW("MD5 hash already finalized");
    if (n && !MD5_Update(&p->ctx, s, n)) THROW("Error updating MD5 hash");
  }


  void MD5::getDigest(unsigned char digest[16]) {
    finalize();
    for (int i = 0; i < 16; i++) digest[i] = this->digest[i];
  }


  string MD5::toString() const {
    if (!finalized) THROW("MD5 hash not finalized");

    string result;
    for (int i = 0; i < 16; i++) result += String::printf("%02x", digest[i]);
    return result;
  }


  string MD5::toString() {
    finalize();
    return const_cast<const MD5 *>(this)->toString();
  }


  void MD5::finalize() {
    if (finalized) return;
    if (!MD5_Final(digest, &p->ctx)) THROW("Could not compute MD5 checksum");
    finalized = true;
  }


  void MD5::reset() {
    if (!MD5_Init(&p->ctx)) THROW("Error initializing MD5 hash");
    finalized = false;
  }


  string MD5::hash(const string &s) {
    return MD5(s).toString();
  }
}
