/******************************************************************************\

          This file is part of the C! library.  A.K.A the cbang library.

              Copyright (c) 2003-2012, Cauldron Development LLC
                 Copyright (c) 2003-2012, Stanford University
                             All rights reserved.

        The C! library is free software: you can redistribute it and/or
        modify it under the terms of the GNU Lesser General Public License
        as published by the Free Software Foundation, either version 2.1 of
        the License, or (at your option) any later version.

        The C! library is distributed in the hope that it will be useful,
        but WITHOUT ANY WARRANTY; without even the implied warranty of
        MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
        Lesser General Public License for more details.

        You should have received a copy of the GNU Lesser General Public
        License along with the C! library.  If not, see
        <http://www.gnu.org/licenses/>.

        In addition, BSD licensing may be granted on a case by case basis
        by written permission from at least one of the copyright holders.
        You may request written permission by emailing the authors.

                For information regarding this software email:
                               Joseph Coffland
                        joseph@cauldrondevelopment.com

\******************************************************************************/

#ifndef CBANG_RSAKEY_H
#define CBANG_RSAKEY_H

#include "KeyGenCallback.h"

#include <cbang/SmartPointer.h>

#include <string>
#include <ostream>

typedef struct rsa_st RSA;

namespace cb {
  class RSAKey {
    RSA *key;
    bool pub;

  public:
    typedef enum {
      NO_PADDING = 100000,
      PKCS1_PADDING,
      SSLV23_PADDING,
      PKCS1_OAEP_PADDING,
      X931_PADDING,
    } padding_t;

    RSAKey() : key(0), pub(true) {}
    RSAKey(const std::string &pem, bool pub = true);
    ~RSAKey();

    bool isValid() const {return key;}
    bool isPublic() const {return pub;}
    unsigned size() const;

    void generate(unsigned size, unsigned long e = 65537,
                  SmartPointer<KeyGenCallback> callback = 0);
    unsigned encrypt(const void *from, void *to,
                     int padding = PKCS1_PADDING) const;
    unsigned decrypt(const void *from, void *to,
                     int padding = PKCS1_PADDING) const;

    void release();

    static std::string getErrorStr();

    std::ostream &print(std::ostream &stream) const;
  };

  inline static
  std::ostream &operator<<(std::ostream &stream, const RSAKey &k) {
    return k.print(stream);
  }
}

#endif // CBANG_RSAKEY_H

