/******************************************************************************\

          This file is part of the C! library.  A.K.A the cbang library.

              Copyright (c) 2003-2012, Cauldron Development LLC
                 Copyright (c) 2003-2012, Stanford University
                             All rights reserved.

        The C! library is free software: you can redistribute it and/or
        modify it under the terms of the GNU Lesser General Public License
        as published by the Free Software Foundation, either version 2.1 of
        the License, or (at your option) any later version.

        The C! library is distributed in the hope that it will be useful,
        but WITHOUT ANY WARRANTY; without even the implied warranty of
        MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
        Lesser General Public License for more details.

        You should have received a copy of the GNU Lesser General Public
        License along with the C! library.  If not, see
        <http://www.gnu.org/licenses/>.

        In addition, BSD licensing may be granted on a case by case basis
        by written permission from at least one of the copyright holders.
        You may request written permission by emailing the authors.

                For information regarding this software email:
                               Joseph Coffland
                        joseph@cauldrondevelopment.com

\******************************************************************************/

#ifndef CBANG_ORDERED_DICT_H
#define CBANG_ORDERED_DICT_H

#include <map>
#include <vector>
#include <string>

#include <cbang/Exception.h>

namespace cb {
  template <typename T>
  class OrderedDict : public std::vector<std::pair<std::string, T> > {
    typedef T type_t;
    typedef std::vector<std::pair<std::string, type_t> > vector_t;
    typedef std::map<std::string, typename vector_t::size_type> dict_t;
    dict_t dict;

  public:
    void clear() {
      vector_t::clear();
      dict.clear();
    }

    typename vector_t::size_type indexOf(const std::string &key) const {
      typename dict_t::const_iterator it = dict.find(key);
      if (it == dict.end()) CBANG_THROWS("Key '" << key << "' not found");
      return it->second;
    }


    bool has(const std::string &key) const {
      return dict.find(key) != dict.end();
    }


    bool insert(const std::string &key, const type_t &value) {
      typename dict_t::const_iterator it = dict.find(key);
      if (it == dict.end()) {
        dict.insert(typename dict_t::value_type(key,  vector_t::size()));
        vector_t::push_back(typename vector_t::value_type(key, value));

        return true;
      }

      this->at(it->second) = typename vector_t::value_type(key, value);

      return false;
    }


    typename OrderedDict::type_t &
    operator[](typename vector_t::size_type i) {
      if (vector_t::size() <= i) CBANG_THROWS("Index " << i << " out of range");
      return this->at(i).second;
    }


    const typename OrderedDict::type_t &
    operator[](typename vector_t::size_type i) const {
      if (vector_t::size() <= i) CBANG_THROWS("Index " << i << " out of range");
      return this->at(i).second;
    }


    typename OrderedDict::type_t &operator[](const std::string &key) {
      typename dict_t::const_iterator it = dict.find(key);
      if (it == dict.end()) {
        dict[key] = vector_t::size();
        vector_t::push_back(typename vector_t::value_type(key, T()));
        return vector_t::back().second;
      }

      return this->at(it->second).second;
    }


    const typename OrderedDict::type_t &
    operator[](const std::string &key) const {
      typename dict_t::const_iterator it = dict.find(key);
      if (it == dict.end()) CBANG_THROWS("Key '" << key << "' not found");
      if (vector_t::size() <= it->second)
        CBANG_THROWS("Key '" << key << "' index " << it->second << " out of range.");

      return this->at(it->second).second;
    }
  };
}

#endif // CBANG_ORDERED_DICT_H

