/******************************************************************************\

    OpenSCAM is an Open-Source CAM software.
    Copyright (C) 2011-2012 Joseph Coffland <joseph@cauldrondevelopment.com>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

\******************************************************************************/

#include "Move.h"

#include <cbang/Math.h>
#include <cbang/log/Logger.h>

using namespace OpenSCAM;
using namespace cb;
using namespace std;


Move::Move(MoveType type, const Axes &start, const Axes &end, real startTime,
           unsigned tool, real feed, real speed, unsigned line) :
  Segment3R(start.getVector3D(), end.getVector3D()), type(type),
  start(start), end(end), tool(tool), feed(feed), speed(speed), line(line),
  dist(start.distance(end)), time(dist / feed * 60), startTime(startTime) {
  if (!feed) THROW("Zero feed");
}


#if 0
real Move::getToolRadius() const {
  real radius = tool.getRadius();
  if (radius <= 0) THROWS("Invalid tool radius line=" << line << ' ' << tool);
  return radius;
}


real Move::getToolLength() const {
  real length = tool.getLength();
  if (length <= 0) THROWS("Invalid tool length line=" << line << ' ' << tool);
  return length;
}


Rectangle3R Move::getBounds(real tolerance) const {
  real radius = getToolRadius();
  real length = getToolLength();
  const Vector3R &p1 = getStartPt();
  const Vector3R &p2 = getEndPt();

  real minX = std::min(p1.x(), p2.x()) - radius - tolerance;
  real minY = std::min(p1.y(), p2.y()) - radius - tolerance;
  real minZ = std::min(p1.z(), p2.z()) - tolerance;
  real maxX = std::max(p1.x(), p2.x()) + radius + tolerance;
  real maxY = std::max(p1.y(), p2.y()) + radius + tolerance;
  real maxZ = std::max(p1.z(), p2.z()) + length + tolerance;

  return Rectangle3R(Vector3R(minX, minY, minZ), Vector3R(maxX, maxY, maxZ));
}
#endif

Color Move::getColor() const {
  switch (type) {
  case MoveType::MOVE_RAPID:   return Color::RED;
  case MoveType::MOVE_CUTTING: return Color::GREEN;
  case MoveType::MOVE_PROBE:   return Color::BLUE;
  case MoveType::MOVE_DRILL:   return Color::YELLOW;
  }
  THROWS("Invalid move type " << type);
}


Vector3R Move::getEndPtAtTime(real time) const {
  if (getEndTime() <= time) return getEndPt();
  if (time <= getStartTime()) return getStartPt();
  real delta = time - getStartTime();
  return getStartPt() + (getEndPt() - getStartPt()) * delta / getTime();
}


void Move::print(std::ostream &stream) const {
  stream
    << "type:" << type << ' '
    << "x:" << end.getX() << ' '
    << "y:" << end.getY() << ' '
    << "z:" << end.getZ() << ' '
    << "tool:" << tool << ' '
    << "feed:" << feed << ' '
    << "speed:" << speed << ' '
    << "line:" << line;
}
