/******************************************************************************\

    OpenSCAM is an Open-Source CAM software.
    Copyright (C) 2011-2012 Joseph Coffland <joseph@cauldrondevelopment.com>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

\******************************************************************************/

#ifndef OPENSCAM_PROJECT_H
#define OPENSCAM_PROJECT_H

#include "ResolutionMode.h"

#include <cbang/config/OptionProxy.h>
#include <cbang/util/OrderedDict.h>

#include <openscam/Geom.h>


namespace OpenSCAM {
  class ToolTable;

  class Project {
    cb::OptionProxy options;
    std::string filename;

    ResolutionMode resolutionMode;
    double resolution;

    ToolTable &toolTable;

    double workpieceMargin;
    std::string workpieceMin;
    std::string workpieceMax;

    typedef cb::OrderedDict<uint64_t> files_t;
    files_t files;
    bool watch;
    uint64_t lastWatch;

  public:
    Project(cb::Options &options, ToolTable &toolTable,
            const std::string &filename = std::string());

    const std::string &getFilename() const {return filename;}
    void setFilename(const std::string &filename);

    std::string makeRelative(const std::string &path) const;
    std::string makeAbsolute(const std::string &path) const;

    ResolutionMode getResolutionMode() const {return resolutionMode;}
    void setResolutionMode(ResolutionMode x) {resolutionMode = x;}
    double getResolution() const {return resolution;}
    void setResolution(double x) {options["resolution"].set(x);}

    ToolTable &getToolTable() {return toolTable;}

    void reset();
    void load(const std::string &filename);
    void save(const std::string &filename = std::string());

    void addFile(const std::string &filename);
    std::vector<std::string> getRelativeFiles() const;
    std::vector<std::string> getAbsoluteFiles() const;
    void setFiles(std::vector<std::string> &files);

    bool getAutomaticWorkpiece() const;
    void setAutomaticWorkpiece(bool x)
    {return options["automatic-workpiece"].set(x);}
    double getWorkpieceMargin() const {return workpieceMargin;}
    void setWorkpieceMargin(double x)
    {return options["workpiece-margin"].set(x);}
    void setWorkpieceBounds(const cb::Rectangle3D &bounds);
    cb::Rectangle3D getWorkpieceBounds() const;

    bool checkFiles();
  };
}

#endif // OPENSCAM_PROJECT_H

