/******************************************************************************\

    OpenSCAM is an Open-Source CAM software.
    Copyright (C) 2011-2012 Joseph Coffland <joseph@cauldrondevelopment.com>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

\******************************************************************************/

#ifndef OPENSCAM_SIMULATION_MANAGER_H
#define OPENSCAM_SIMULATION_MANAGER_H

#include <openscam/sim/Machine.h>
#include <openscam/gcode/Parser.h>
#include <openscam/gcode/Interpreter.h>

#include <cbang/SmartPointer.h>


namespace cb {class Options;}

namespace OpenSCAM {
  class ToolPath;
  class CutWorkpiece;
  class Project;

  class SimulationManager : public Machine {
    Parser parser;
    Controller controller;

    cb::SmartPointer<ToolPath> path;
    cb::SmartPointer<CutWorkpiece> cutWP;

  public:
    SimulationManager(cb::Options &options);
    ~SimulationManager();

    Controller &getController() {return controller;}
    const Controller &getController() const {return controller;}
    ToolTable &getToolTable() {return controller.getToolTable();}

    cb::SmartPointer<ToolPath> getToolPath() const {return path;}
    cb::SmartPointer<CutWorkpiece> getCutWorkpiece() const {return cutWP;}

    void init();

    void reset();
    void load(Project &project);

    // From Machine
    void move(const Move &move);
  };
}

#endif // OPENSCAM_SIMULATION_MANAGER_H

