/******************************************************************************\

    OpenSCAM is an Open-Source CAM software.
    Copyright (C) 2011-2012 Joseph Coffland <joseph@cauldrondevelopment.com>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

\******************************************************************************/

#ifndef OPENSCAM_FILTER_H
#define OPENSCAM_FILTER_H

#include <openscam/Geom.h>
#include <openscam/gcode/Printer.h>

#include <openscam/sim/Controller.h>
#include <openscam/gcode/Interpreter.h>

#include <cbang/io/Reader.h>

#include <string>
#include <set>
#include <istream>


namespace cb {class Options;}

namespace OpenSCAM {
  class Filter : public Machine, public Printer, public cb::Reader {
    cb::Options &options;

    Controller controller;
    Interpreter interp;

    std::string f;

    char var;
    std::string op;
    double value;

    typedef std::set<cb::Segment3D> moves_t;
    moves_t moves;
    bool uniqueMove;
    bool moved;

  public:
    Filter(cb::Options &options, std::ostream &stream,
           const std::string &f = std::string());

    void init();

    // From cb::Reader
    void read(const cb::InputSource &source);

    // From Machine
    void move(const Move &move);

    // From Processor
    void operator()(const cb::SmartPointer<Block> &block);
  };
}

#endif // OPENSCAM_FILTER_H
