/******************************************************************************\

    OpenSCAM is an Open-Source CAM software.
    Copyright (C) 2011-2012 Joseph Coffland <joseph@cauldrondevelopment.com>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

\******************************************************************************/

#ifndef OPENSCAM_BUILDER_H
#define OPENSCAM_BUILDER_H

#include <cbang/Exception.h>

#include <gtk/gtk.h>


namespace OpenSCAM {
  class Builder {
    GtkBuilder *builder;

  public:
    Builder(GtkBuilder *builder) : builder(builder) {}
    ~Builder() {g_object_unref(builder);}

    GtkBuilder *getWidget() {return builder;}
    void addFromString(const char *data, unsigned length);

    void connect(void *data);
    void connect(GObject *object, const std::string &signal, GCallback callback,
                 void *data = 0);
    void connect(const std::string &object, const std::string &signal,
                 GCallback callback, void *data = 0);

    template <typename T>
    T *get(const std::string &name) const {
      GObject *object = gtk_builder_get_object(builder, name.c_str());
      if (!object) THROWS("Object " << name << " not found");
      return (T *)object;
    }
  };
}

#endif // OPENSCAM_BUILDER_H

