/******************************************************************************\

    OpenSCAM is an Open-Source CAM software.
    Copyright (C) 2011-2012 Joseph Coffland <joseph@cauldrondevelopment.com>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

\******************************************************************************/

#include "Dialog.h"

#include "GTKHelper.h"
#include "Builder.h"

#include <cbang/log/Logger.h>
#include <cbang/SStream.h>

using namespace cb;
using namespace OpenSCAM;

GTK_SIGNAL1V(on_ok, GtkWidget *, Dialog);
GTK_SIGNAL1V(on_cancel, GtkWidget *, Dialog);
GTK_SIGNAL2B(on_delete, GtkWidget *, GdkEvent *, Dialog);
GTK_SIGNAL2V(on_response, GtkDialog *, gint, Dialog);


Dialog::Dialog(Builder &builder, const char *name) :
  name(name), dialog(builder.get<GtkDialog>(SSTR(name << "_dialog"))) {

  builder.connect(SSTR(name << "_dialog"), "delete-event",
                  G_CALLBACK(::on_delete), this);
  builder.connect(SSTR(name << "_dialog"), "close",
                  G_CALLBACK(::on_cancel), this);
  builder.connect(SSTR(name << "_dialog"), "response",
                  G_CALLBACK(::on_response), this);
}


void Dialog::setTitle(const std::string &title) {
  gtk_window_set_title((GtkWindow *)dialog, title.c_str());
}


void Dialog::setTransientFor(GtkWindow *parent) {
  gtk_window_set_transient_for(GTK_WINDOW(dialog), parent);
}


void Dialog::show() {
  gtk_window_present((GtkWindow *)dialog);
}


void Dialog::hide() {
  gtk_widget_hide((GtkWidget *)dialog);
}


int Dialog::run() {
  return gtk_dialog_run(dialog);
}


void Dialog::accept() {
  LOG_WARNING("Dialog::accept() " << name << " not yet implemented");
  hide();
}


void Dialog::cancel() {
  hide();
}


void Dialog::response(int id) {
  switch (id) {
  case GTK_RESPONSE_ACCEPT:
  case GTK_RESPONSE_YES:
  case GTK_RESPONSE_OK:
    accept();
    break;

  case GTK_RESPONSE_REJECT:
  case GTK_RESPONSE_NO:
  case GTK_RESPONSE_CANCEL:
  case GTK_RESPONSE_CLOSE:
    cancel();
    break;

  default: break;
  }
}


void Dialog::on_ok(GtkWidget *widget) {
  accept();
}


void Dialog::on_cancel(GtkWidget *widget) {
  cancel();
}


bool Dialog::on_delete(GtkWidget *widget, GdkEvent *event) {
  cancel();
  return true;
}


void Dialog::on_response(GtkDialog *dialog, gint id) {
  response(id);
}
