/******************************************************************************\

    OpenSCAM is an Open-Source CAM software.
    Copyright (C) 2011-2012 Joseph Coffland <joseph@cauldrondevelopment.com>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

\******************************************************************************/

#include "FileDialog.h"

#include "GUIApp.h"
#include "Builder.h"

#include <cbang/os/SystemUtilities.h>

using namespace std;
using namespace cb;
using namespace OpenSCAM;


FileDialog::FileDialog(GUIApp &app) :
  Dialog(*app.getBuilder(), "file"), app(app) {

  gtk_dialog_add_buttons(dialog, GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
                         GTK_STOCK_OK, GTK_RESPONSE_ACCEPT, (void *)0);
}


void FileDialog::setFilename(const string &path) {
  filename = SystemUtilities::makeAbsolute(path); // Save it
}


void FileDialog::setCurrentFolder(const std::string &path) {
  string dir;

  if (path.empty()) dir = SystemUtilities::getcwd();
  else if (SystemUtilities::isDirectory(path)) dir = path;
  else dir = SystemUtilities::dirname(path);

  gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(dialog), dir.c_str());
}


string FileDialog::getFilename() const {
  char *filename;
  filename = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(dialog));
  string _filename = filename;
  g_free(filename);
  return _filename;
}


void FileDialog::confirmOverwrite(bool x) {
  gtk_file_chooser_set_do_overwrite_confirmation(GTK_FILE_CHOOSER(dialog), x);
}


void FileDialog::setFilter(const string &name, const string &patterns) {
  clearFilters();
  if (!patterns.empty()) addFilter(name, patterns);
}


void FileDialog::addFilter(const string &name, const string &patterns) {
  vector<string> tokens;
  String::tokenize(patterns, tokens);

  GtkFileFilter *filter = gtk_file_filter_new();
  if (!name.empty()) gtk_file_filter_set_name(filter, name.c_str());

  for (unsigned i = 0; i < tokens.size(); i++)
    gtk_file_filter_add_pattern(filter, tokens[i].c_str());

  gtk_file_chooser_add_filter((GtkFileChooser *)dialog, filter);
}


void FileDialog::clearFilters() {
  GSList *list = gtk_file_chooser_list_filters((GtkFileChooser *)dialog);

  for (GSList *node = list; node; node = list->next)
    gtk_file_chooser_remove_filter((GtkFileChooser *)dialog,
                                   GTK_FILE_FILTER(node->data));

  g_slist_free(list);
}


bool FileDialog::open(GtkWindow *parent, const string &name,
                      const string &filters) {
  setTransientFor(parent);
  gtk_file_chooser_set_action((GtkFileChooser *)dialog,
                              GTK_FILE_CHOOSER_ACTION_OPEN);
  setFilter(name, filters);
  setCurrentFolder(filename);

  bool pass = run() == GTK_RESPONSE_ACCEPT;

  return pass;
}


bool FileDialog::save(GtkWindow *parent, const string &name,
                      const string &filters) {
  setTransientFor(parent);
  confirmOverwrite(true);
  gtk_file_chooser_set_action((GtkFileChooser *)dialog,
                              GTK_FILE_CHOOSER_ACTION_SAVE);
  setFilter(name, filters);
  setCurrentFolder(filename);
  if (!filename.empty()) {
    string name = SystemUtilities::basename(filename);
    gtk_file_chooser_set_current_name(GTK_FILE_CHOOSER(dialog), name.c_str());
  }

  bool pass = run() == GTK_RESPONSE_ACCEPT;

  return pass;
}


void FileDialog::accept() {
  hide();
}
