/******************************************************************************\

    OpenSCAM is an Open-Source CAM software.
    Copyright (C) 2011-2012 Joseph Coffland <joseph@cauldrondevelopment.com>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

\******************************************************************************/

#include "GtkObjectSet.h"

#include "Builder.h"

#include <cbang/String.h>
#include <cbang/Exception.h>

#include <gtk/gtk.h>
#include <string.h>

using namespace std;
using namespace cb;
using namespace OpenSCAM;


GtkObjectSet::GtkObjectSet(Builder &builder, const std::string &suffix) {
  GSList *start = gtk_builder_get_objects(builder.getWidget());

  for (GSList *list = start; list; list = list->next) {
    if (!GTK_IS_BUILDABLE(list->data)) continue;

    const char *name = gtk_buildable_get_name((GtkBuildable *)list->data);
    if (String::endsWith(name, suffix))
      insert(value_type(string(name, strlen(name) - suffix.length()),
                        (GObject *)list->data));
  }

  g_slist_free(start);
}


GObject *GtkObjectSet::get(const string &name) const {
  const_iterator it = find(name);
  if (it == end()) THROWS("GtkObject '" << name << "' not found");
  return it->second;
}
