/******************************************************************************\

    OpenSCAM is an Open-Source CAM software.
    Copyright (C) 2011-2012 Joseph Coffland <joseph@cauldrondevelopment.com>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

\******************************************************************************/

#ifndef OPENSCAM_PROJECT_DIALOG_H
#define OPENSCAM_PROJECT_DIALOG_H

#include "Dialog.h"
#include "ToolDialog.h"


namespace OpenSCAM {
  class GUIApp;
  class Builder;
  class Project;
  class Tool;

  class ProjectDialog : public Dialog {
    GUIApp &app;
    ToolDialog toolDialog;

    bool loading;
    GtkComboBox *resolutionCombo;
    GtkSpinButton *resolutionSpin;

    GtkTreeView *ncFileTree;
    GtkListStore *ncFileStore;

    GtkTreeView *toolTableTree;
    GtkListStore *toolTableStore;

    GtkWidget *workpieceTable;
    GtkRadioButton *autoWorkpieceRadio;
    GtkRadioButton *cuboidWorkpieceRadio;

    GtkSpinButton *wpMarginSpin;
    GtkSpinButton *minXSpin;
    GtkSpinButton *minYSpin;
    GtkSpinButton *minZSpin;
    GtkSpinButton *maxXSpin;
    GtkSpinButton *maxYSpin;
    GtkSpinButton *maxZSpin;

    cb::SmartPointer<Project> project;
    bool newTool;
    GtkTreeIter currentTool;

  public:
    ProjectDialog(GUIApp &app);
    ~ProjectDialog();

    bool getAutomaticWorkpiece();
    void setAutomaticWorkpiece(bool x);

    void load(const cb::SmartPointer<Project> &project);
    void edit(const cb::SmartPointer<Project> &project);
    void addTool(const cb::SmartPointer<Tool> &tool);
    void saveTool();
    void storeTool(GtkTreeIter iter, Tool &tool);

    void removeListSelection(GtkTreeView *tree);

    void on_resolution_changed(GtkWidget *widget);
    void on_tool_table_row_activated(GtkTreeView *tree, GtkTreePath *path,
                                     GtkTreeViewColumn *column);
    void on_nc_files_add(GtkWidget *widget);
    void on_nc_files_remove(GtkWidget *widget);
    void on_nc_file_edited(GtkWidget *widget, char *path, char *text);
    void on_tool_table_add(GtkWidget *widget);
    void on_tool_table_remove(GtkWidget *widget);

    // From Dialog
    void accept();
  };
}

#endif // OPENSCAM_PROJECT_DIALOG_H

