/******************************************************************************\

    OpenSCAM is an Open-Source CAM software.
    Copyright (C) 2011-2012 Joseph Coffland <joseph@cauldrondevelopment.com>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

\******************************************************************************/

#ifndef OPENSCAM_TOOL_DIALOG_H
#define OPENSCAM_TOOL_DIALOG_H

#include "Dialog.h"

#include <openscam/sim/ToolShape.h>
#include <openscam/sim/ToolUnits.h>

#include <cbang/SmartPointer.h>

#include <string>


namespace OpenSCAM {
  class GUIApp;
  class Builder;
  class Tool;

  class ToolDialog : public Dialog {
    GUIApp &app;

    GtkSpinButton *numberSpin;
    GtkComboBox *unitsCombo;
    GtkComboBox *shapeCombo;
    GtkSpinButton *lengthSpin;
    GtkSpinButton *diameterSpin;
    GtkSpinButton *snubDiameterSpin;
    GtkLabel *snubDiameterLabel;
    GtkEntry *descriptionEntry;
    GtkImage *toolImage;

    bool busy;
    ToolUnits lastUnits;

    const unsigned width;
    const unsigned height;
    GdkPixmap *pixmap;

  public:
    ToolDialog(GUIApp &app);

    unsigned getNumber();
    void setNumber(unsigned number);

    ToolUnits getUnits();
    void setUnits(ToolUnits units);

    ToolShape getShape();
    void setShape(ToolShape shape);

    double getLength();
    void setLength(double length);

    double getDiameter();
    void setDiameter(double diameter);

    double getSnubDiameter();
    void setSnubDiameter(double diameter);

    std::string getDescription();
    void setDescription(const std::string &description);

    void on_tool_units_changed(GtkWidget *widget);
    void on_tool_changed(GtkWidget *widget);

    // From Dialog
    void accept();
  };
}

#endif // OPENSCAM_TOOL_DIALOG_H

