/******************************************************************************\

    OpenSCAM is an Open-Source CAM software.
    Copyright (C) 2011-2012 Joseph Coffland <joseph@cauldrondevelopment.com>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

\******************************************************************************/

#ifndef OPENSCAM_AXES_H
#define OPENSCAM_AXES_H

#include <openscam/Geom.h>

#include <cbang/Exception.h>

#include <cctype>
#include <ostream>

#include <string.h>


namespace OpenSCAM {
  class Axes : public cb::Vector<9, real> {
  public:
    static const char *AXES;

    Axes() {}
    Axes(const cb::Vector<9, real> &v) : cb::Vector<9, real>(v) {}

    void reset() {memset(data, 0, sizeof(data));}

    double get(char c) const {return getIndex(toIndex(c));}
    void set(char c, double value) {setIndex(toIndex(c), value);}
    double getIndex(unsigned i) const {return data[i];}
    void setIndex(unsigned i, double value) {data[i] = value;}

    double getX() const {return data[0];}
    double getY() const {return data[1];}
    double getZ() const {return data[2];}
    double getA() const {return data[3];}
    double getB() const {return data[4];}
    double getC() const {return data[5];}
    double getU() const {return data[6];}
    double getV() const {return data[7];}
    double getW() const {return data[8];}

    void setX(double value) {data[0] = value;}
    void setY(double value) {data[1] = value;}
    void setZ(double value) {data[2] = value;}
    void setA(double value) {data[3] = value;}
    void setB(double value) {data[4] = value;}
    void setC(double value) {data[5] = value;}
    void setU(double value) {data[6] = value;}
    void setV(double value) {data[7] = value;}
    void setW(double value) {data[8] = value;}

    cb::Vector2D getVector2D() const {return cb::Vector2D(data[0], data[1]);}
    cb::Vector3D getVector3D() const
    {return cb::Vector3D(data[0], data[1], data[2]);}

    double distance(const Axes &o) const;

    std::ostream &print(std::ostream &stream) const;

    inline static unsigned toIndex(char axis) {
      switch (std::toupper(axis)) {
      case 'X': return 0;
      case 'Y': return 1;
      case 'Z': return 2;
      case 'A': return 3;
      case 'B': return 4;
      case 'C': return 5;
      case 'U': return 6;
      case 'V': return 7;
      case 'W': return 8;
      default: THROWS("Invalid axis " << axis);
      }
    }

    
    inline static char toAxis(unsigned i) {
      switch (i) {
      case 0: return 'X';
      case 1: return 'Y';
      case 2: return 'Z';
      case 3: return 'A';
      case 4: return 'B';
      case 5: return 'C';
      case 6: return 'U';
      case 7: return 'V';
      case 8: return 'W';
      default: THROWS("Invalid axis index " << i);
      }
    }
  };

  inline static
  std::ostream &operator<<(std::ostream &stream, const Axes &a) {
    return a.print(stream);
  }
}

#endif // OPENSCAM_AXES_H

