/******************************************************************************\

    OpenSCAM is an Open-Source CAM software.
    Copyright (C) 2011-2012 Joseph Coffland <joseph@cauldrondevelopment.com>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

\******************************************************************************/

#ifndef OPENSCAM_MACHINE_H
#define OPENSCAM_MACHINE_H

#include "Axes.h"

#include <openscam/cutsim/MoveType.h>


namespace OpenSCAM {
  class Move;

  class Machine {
    double time;
    double distance;

    // TODO Load machine configuration, ramp up/down, rapid feed, etc.
    double feed;
    double speed;
    unsigned tool;

    bool mistCoolant;
    bool floodCoolant;

    Axes position;

  public:
    Machine() :
      time(0), distance(0), feed(0), speed(0), tool(1), mistCoolant(false),
      floodCoolant(false) {}
    virtual ~Machine() {} // Compiler needs this

    double getTime() const {return time;}
    double getDistance() const {return distance;}

    virtual void setFeed(double feed) {this->feed = feed;}
    virtual double getFeed() const {return feed;}
    virtual void setSpeed(double speed) {this->speed = speed;}
    virtual double getSpeed() const {return speed;}
    virtual void setTool(unsigned tool) {this->tool = tool;}
    virtual unsigned getTool() const {return tool;}
    virtual void setMistCoolant(bool x) {mistCoolant = x;}
    virtual bool getMistCoolant() const {return floodCoolant;}
    virtual void setFloodCoolant(bool x) {floodCoolant = x;}
    virtual bool getFloodCoolant() const {return floodCoolant;}

    const Axes &getPosition() const {return position;}

    void makeMove(MoveType type, const Axes &pos);

    virtual void start();
    virtual void move(const Move &move);
    virtual void dwell(double time);
    virtual void end() {}
  };
}

#endif // OPENSCAM_MACHINE_H
