/******************************************************************************\

    OpenSCAM is an Open-Source CAM software.
    Copyright (C) 2011-2012 Joseph Coffland <joseph@cauldrondevelopment.com>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

\******************************************************************************/

#include "Cylinder.h"

#include "Circle.h"

#include <openscam/view/GL.h>

#include <cbang/log/Logger.h>

using namespace cb;
using namespace OpenSCAM;


Cylinder::Cylinder(real radius, real height, Vector3R p) :
  p(p), radius(radius), height(height), quad(gluNewQuadric()) {}


Cylinder::~Cylinder() {
  gluDeleteQuadric((GLUquadric *)quad);
}


Vector3R Cylinder::closest(const Vector3R &q) {
  // Project on to center line segment
  real t = (q - p).dotProduct(Vector3R(0, 0, height)) / (height * height);
  real z = q.z();

  // Above or below
  if (t < 0) { // Below
    real h = p.z() - q.z();
    real sqDist = q.distanceSquared(p);
    real l = sqrt(sqDist - h * h);

    if (l < radius) return Vector3R(q.x(), q.y(), p.z());
    z = p.z();

  } else if (1 < t) { // Above
    z = p.z() + height;
    real h = q.z() - z;
    real sqDist = q.distanceSquared(Vector3R(p.x(), p.y(), z));
    real l = sqrt(sqDist - h * h);

    if (l < radius) return Vector3R(q.x(), q.y(), z);
  }

  // To the side
  Vector2R i(p.x(), p.y());
  Vector2R j(q.x(), q.y());
  if (i == j) return Vector3R(i.x(), i.y(), z);
  Vector2R k = i + (j - i).normalize() * radius;

  return Vector3R(k.x(), k.y(), z);
}


#define WIRE
//#define SOLID

void Cylinder::draw() {
  glMatrixMode(GL_MODELVIEW);
  glPushMatrix();

  glTranslatef(p.x(), p.y(), p.z());
#ifdef SOLID
  gluCylinder((GLUquadric *)quad, radius, radius, height, 100, 100);

  gluDisk((GLUquadric *)quad, 0, radius, 100, 10);
#endif

#ifdef WIRE
  glPushAttrib(GL_CURRENT_BIT);
  glColor4f(0, 0, 0, 1); // Black
  Circle(radius).draw();
  glPopAttrib();
#endif

  glTranslatef(0, 0, height);

#ifdef SOLID
  gluDisk((GLUquadric *)quad, 0, radius, 100, 10);
#endif

#ifdef WIRE
  glPushAttrib(GL_CURRENT_BIT);
  glColor4f(0, 0, 0, 1); // Black
  Circle(radius).draw();
  glPopAttrib();
#endif

  glPopMatrix();
}
