/******************************************************************************\

    OpenSCAM is an Open-Source CAM software.
    Copyright (C) 2011-2012 Joseph Coffland <joseph@cauldrondevelopment.com>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

\******************************************************************************/

#ifndef OPENSCAM_GLUT_APP_H
#define OPENSCAM_GLUT_APP_H

#include <openscam/value/ValueSet.h>

#include <cbang/StdTypes.h>
#include <cbang/SmartPointer.h>
#include <cbang/time/Timer.h>
#include <cbang/Application.h>
#include <cbang/geom/Vector.h>


namespace OpenSCAM {
  class ViewPort;

  class GlutApp : public cb::Application {
    static GlutApp *singleton;

    cb::SmartPointer<ViewPort> viewPort;

    bool fullscreen;
    bool visible;
    bool dirty;

    int formerX;
    int formerY;
    int formerWidth;
    int formerHeight;

    cb::Vector2I mousePosition;
    bool mouseLeftDragging;
    bool mouseMiddleDragging;
    bool mouseRightDragging;

    cb::Timer renderTimer;
    cb::Timer idleTimer;

  public:
    GlutApp(const std::string &name);
    ~GlutApp();

    static GlutApp &instance();

    const cb::SmartPointer<ViewPort> &getViewPort() const {return viewPort;}

    const cb::Vector2I &getMousePosition() const {return mousePosition;}
    bool getLeftMouseDragging() {return mouseLeftDragging;}
    bool getMiddleMouseDragging() {return mouseMiddleDragging;}
    bool getRightMouseDragging() {return mouseRightDragging;}

    // From Application
    int init(int argc, char *argv[]);
    void run();

    void reload();
    void redisplay() {dirty = true;}
    void toggleFullscreen();
    void quit();

    virtual void mouse(int button, int state, int x, int y);
    virtual void motion(int x, int y);
    virtual void keys(unsigned char key, int x, int y);
    virtual void special(int key, int x, int y);
    virtual void render();
    virtual void draw() = 0;
    virtual void resize(unsigned width, unsigned height);
    virtual void visibility(int state);
    virtual void idle();
  };
}

#endif // OPENSCAM_GLUT_APP_H

