#include "Context.h"

#include "Environment.h"

#include "ast/Declaration.h"

using namespace cb;
using namespace OpenSCAM::TPL;


void Context::pushEnv(const Environment &env) {
  if (envStacks.size() <= env.getID()) envStacks.resize(env.getID() + 1);
  envStacks[env.getID()].push_back(new VectorValue(env.getFrameSize()));

  envIDs.push_back(env.getID());
}


void Context::popEnv() {
  unsigned envID = envIDs.back();
  envIDs.pop_back();

  envStacks[envID].pop_back();
}


const SmartPointer<VectorValue> &Context::getCurrentFrame() {
  if (envIDs.empty()) THROW("Empty stack");
  return envStacks[envIDs.back()].back();
}


SmartPointer<Value> &Context::getReference(const Declaration &decl) {
#ifdef DEBUG
  unsigned envID = decl.getEnvID();
  unsigned index = decl.getIndex();

  if (envID < 0) THROWS("Declaration's environment id not set");
  if (index < 0) THROWS("Declaration's environment index not set");

  if (envStacks.size() <= envID) THROWS("Unknown environment ID=" << envID);
  const stack_t &stack = envStacks[envID];

  if (stack.empty()) THROW("Environment frame stack empty");
  VectorValue &frame = *stack.back();

  if (frame.size() <= index)
    THROWS("Declaration's index " << index
           << " is out of range for stack frame " << envID << " with size "
           << frame.size());

  return frame[index];

#else // DEBUG
  return (*envStacks[decl.getEnvID()].back())[decl.getIndex()];
#endif
}
