#ifndef OPENSCAM_TPL_CONTEXT_H
#define OPENSCAM_TPL_CONTEXT_H

#include "type/VectorValue.h"

#include "Environment.h"
#include "Loader.h"

#include <cbang/Exception.h>
#include <cbang/SmartPointer.h>

#include <cbang/io/InputSource.h>
#include <cbang/io/OutputSink.h>

#include <vector>
#include <iostream>


namespace OpenSCAM {
  class Controller;

  namespace TPL {
    class Context {
      Controller &controller;
      cb::SmartPointer<Environment> env;
      cb::SmartPointer<Loader> loader;
      cb::OutputSink sink;
      cb::InputSource source;

      typedef std::vector<cb::SmartPointer<VectorValue> > stack_t;
      typedef std::vector<stack_t> env_stacks_t;
      typedef std::vector<unsigned> env_ids_t;

      env_stacks_t envStacks;
      env_ids_t envIDs;

    public:
      Context(Controller &controller,
              const cb::SmartPointer<Environment> &env = new Environment,
              const cb::SmartPointer<Loader> &loader = new Loader,
              const cb::OutputSink &sink = std::cout,
              const cb::InputSource &source = std::cin) :
        controller(controller), env(env), loader(loader), sink(sink),
        source(source) {}

      Controller &getController() {return controller;}
      const cb::SmartPointer<Environment> &getEnvironment() {return env;}
      const cb::SmartPointer<Loader> &getLoader() {return loader;}

      std::ostream &getOStream() const {return sink.getStream();}
      std::istream &getIStream() const {return source.getStream();}

      void pushEnv(const Environment &env);
      void popEnv();
      const cb::SmartPointer<VectorValue> &getCurrentFrame();
      cb::SmartPointer<Value> &getReference(const Declaration &decl);
    };
  }
}

#endif // OPENSCAM_TPL_CONTEXT_H

