#ifndef OPENSCAM_TPL_ENVIRONMENT_H
#define OPENSCAM_TPL_ENVIRONMENT_H

#include "TPLID.h"
#include "ast/Declaration.h"

#include <cbang/SmartPointer.h>

#include <map>
#include <string>


namespace OpenSCAM {
  namespace TPL {
    class VariableDecl;
    class Name;
    class Value;
    class Module;

    class Environment : public TPLID {
      typedef std::map<std::string, cb::SmartPointer<Declaration> > env_t;
      unsigned id;
      env_t env;
      Environment *parent;
      unsigned nextIndex;
      unsigned children;

    public:
      Environment(Environment *parent = 0) :
        id(0), parent(0), nextIndex(0), children(0)
      {if (parent) setParent(parent);}

      unsigned getID() const {return id;}

      void setParent(Environment *parent);
      Environment *getParent() const {return parent;}
      Environment *getRoot() {return parent ? parent->getRoot() : this;}

      unsigned getFrameSize() const {return nextIndex;}

      void declare(const cb::SmartPointer<Module> &module,
                   const std::string &as = std::string());
      void declare(const std::string &name,
                   const cb::SmartPointer<Declaration> &decl);
      cb::SmartPointer<Declaration> declareVariable(const std::string &name);

      void declareConstant(const std::string &name,
                           const cb::SmartPointer<Value> &value);
      void declareBoolConstant(const std::string &name, bool value);
      void declareIntegerConstant(const std::string &name, int32_t value);
      void declareFloatConstant(const std::string &name, double value);
      void declareStringConstant(const std::string &name,
                                 const std::string &value);

      const cb::SmartPointer<Declaration> &lookup(const Name &name) const;
      const cb::SmartPointer<Declaration> &
      lookup(const std::string &name) const;

    private:
      unsigned getNextID();
    };
  }
}

#endif // OPENSCAM_TPL_ENVIRONMENT_H

