#ifndef OPENSCAM_TPL_INTERPRETER_H
#define OPENSCAM_TPL_INTERPRETER_H

#include "TPLID.h"
#include "Evaluator.h"

#include "ast/ASTInterpreter.h"

#include <cbang/io/InputSource.h>


namespace OpenSCAM {
  namespace TPL {
    class Context;

    class Interpreter : public ASTInterpreter, public TPLID {
      Context &ctx;
      Evaluator evaluator;

    public:
      Interpreter(Context &ctx) : ctx(ctx), evaluator(ctx) {}

      Context &getContext() {return ctx;}

      void interpret(const cb::InputSource &source);

      // From ASTInterpreter
      void interpret(Program &);

      void interpret(ImportDecl &);
      void interpret(ConstantDecl &);
      void interpret(ArgumentDecl &) {}
      void interpret(FunctionDecl &) {}
      void interpret(VariableDecl &) {}

      void interpret(AssignStmt &);
      void interpret(BlockStmt &);
      void interpret(BreakStmt &);
      void interpret(CaseStmt &);
      void interpret(ContinueStmt &);
      void interpret(DoStmt &);
      void interpret(ExpressionStmt &);
      void interpret(ForStmt &);
      void interpret(IfStmt &);
      void interpret(ReturnStmt &);
      void interpret(SwitchStmt &);
      void interpret(WhileStmt &);
    };
  }
}

#endif // OPENSCAM_TPL_INTERPRETER_H

