#include "Loader.h"

#include <openscam/tpl/builtin/STDLibrary.h>
#include <openscam/tpl/builtin/GCodeLibrary.h>
#include <openscam/tpl/builtin/DXFLibrary.h>
#include <openscam/tpl/builtin/MathLibrary.h>

#include <cbang/os/SystemUtilities.h>
#include <cbang/Exception.h>

using namespace std;
using namespace cb;
using namespace OpenSCAM::TPL;


Loader::Loader() {
  if (SystemUtilities::getenv("TPL_PATH")) {
    vector<string> paths;
    String::tokenize(SystemUtilities::getenv("TPL_PATH"), paths, ":");

    string cwd = SystemUtilities::getcwd();

    for (unsigned i = 0; i < paths.size(); i++) {
      string path = paths[i];

      if (path.empty()) continue;

      if (!SystemUtilities::isAbsolute(path))
        path = SystemUtilities::makeRelative(cwd, path);

      this->paths.push_back(path);
    }
  }

  // Builtins
  add(new STDLibrary);
  add(new GCodeLibrary);
  add(new DXFLibrary);
  add(new MathLibrary);
}


void Loader::add(const SmartPointer<Module> &module) {
  string path =
    module->getPath().empty() ? module->getName() : module->getPath();

  if (!modules.insert(modules_t::value_type(path, module)).second)
    THROWS("Module '" << path << "' already added");
}


const SmartPointer<Module> &Loader::find(const string &path) const {
  modules_t::const_iterator it = modules.find(path);
  if (it != modules.end()) return it->second;
  THROWS("Could not find module '" << path << "'");
}


SmartPointer<Module> Loader::load(const string &name, const string &cwd) {
  // Current path
  SmartPointer<Module> module = tryLoad(name, cwd);
  if (!module.isNull()) return module;

  // Import paths
  for (paths_t::iterator it = paths.begin(); it != paths.end(); it++) {
    module = tryLoad(name, *it);
    if (!module.isNull()) return module;
  }

  // Built-ins
  return find(name);
}


SmartPointer<Module> Loader::tryLoad(const string &name, const string &path) {
  string filename = path + "/" + String::replace(name, "\\.", "/") + ".tpl";

  modules_t::iterator it = modules.find(filename);
  if (it != modules.end()) return it->second;

  if (!SystemUtilities::exists(filename)) return 0;

  SmartPointer<Module> module = new Module(name, filename);
  add(module);
  return module;
}
