#ifndef OPENSCAM_TPL_MODULE_H
#define OPENSCAM_TPL_MODULE_H

#include "builtin/BuiltinFunctionPtr.h"
#include "ast/NamespaceDecl.h"
#include "type/Types.h"

#include <cbang/SmartPointer.h>

namespace OpenSCAM {
  namespace TPL {
    class Interpreter;

    class Module : public NamespaceDecl {
      const std::string path;
      bool loaded;
      bool initialized;

    public:
      Module(const std::string &name, const std::string &path = std::string()) :
        NamespaceDecl(name, new Environment, 0, TPLID_MODULE), path(path),
        loaded(path.empty()), initialized(false) {}

      const std::string &getPath() const {return path;}

      bool isLoaded() const {return loaded;}
      void setLoaded(bool loaded) {this->loaded = loaded;}

      void init(Interpreter &interpreter);
    };
  }
}

#define OPENSCAM_TPL_DEFINE_FUNC(name)                                  \
  static cb::SmartPointer<Value> name(Context &ctx, const Arguments &args)

#define OPENSCAM_TPL_DECLARE_FUNC(lib, name, args)                      \
  getEnvironment()->declare(#name,                                      \
                            new BuiltinFunctionPtr(&lib::name, #name, args))

#endif // OPENSCAM_TPL_MODULE_H

