#ifndef OPENSCAM_PRETTY_PRINTER_H
#define OPENSCAM_PRETTY_PRINTER_H

#include "TPLID.h"
#include "ast/ASTNode.h"
#include "ast/ASTVisitor.h"

#include <cbang/SmartPointer.h>
#include <cbang/io/OutputSink.h>

#include <ostream>


namespace OpenSCAM {
  namespace TPL {
    class PrettyPrinter : public TPLID, public ASTVisitor {
      cb::OutputSink sync;
      std::ostream &stream;

      unsigned indent;

    public:
      PrettyPrinter(const cb::OutputSink &sync, unsigned indent = 0) :
        sync(sync), stream(sync.getStream()), indent(indent) {}

      void print(const cb::SmartPointer<ASTNode> &node) {node->visit(*this);}

      // From ASTVisitor
      void visit(Program &node);

      void visit(ImportDecl &node);
      void visit(ConstantDecl &node);
      void visit(ArgumentDecl &node);
      void visit(FunctionDecl &node);

      void visit(AssignStmt &node);
      void visit(BlockStmt &node);
      void visit(BreakStmt &node);
      void visit(CaseStmt &node);
      void visit(ContinueStmt &node);
      void visit(DoStmt &node);
      void visit(ExpressionStmt &node);
      void visit(ForStmt &node);
      void visit(IfStmt &node);
      void visit(ReturnStmt &node);
      void visit(SwitchStmt &node);
      void visit(WhileStmt &node);

      void visit(BinaryExpr &node);
      void visit(CallExpr &node);
      void visit(QuotedExpr &node);
      void visit(SliceExpr &node);
      void visit(UnaryExpr &node);
      void visit(VariableExpr &node);
      void visit(VectorExpr &node);

      void visit(Constant &node);

      void newLine();
    };
  }
}

#endif // OPENSCAM_PRETTY_PRINTER_H

