#include "MathLibrary.h"

#include <cbang/Math.h>

#include <limits>

using namespace std;
using namespace cb;
using namespace OpenSCAM::TPL;


MathLibrary::MathLibrary() : Module("math") {
#define DECLARE_FUNC(name, args)                            \
  OPENSCAM_TPL_DECLARE_FUNC(MathLibrary, name, args)

  DECLARE_FUNC(near, "a, b, delta=0.0000001");
  DECLARE_FUNC(min,  "a, b");
  DECLARE_FUNC(max,  "a, b");

  // math.h
  DECLARE_FUNC(acos,  "x");
  DECLARE_FUNC(asin,  "x");
  DECLARE_FUNC(atan,  "x");
  DECLARE_FUNC(atan2, "y, x");
  DECLARE_FUNC(cos,   "x");
  DECLARE_FUNC(cosh,  "x");
  DECLARE_FUNC(sin,   "x");
  DECLARE_FUNC(sinh,  "x");
  DECLARE_FUNC(tan,   "x");
  DECLARE_FUNC(tanh,  "x");
  DECLARE_FUNC(exp,   "x");
  DECLARE_FUNC(log,   "x");
  DECLARE_FUNC(log10, "x");
  DECLARE_FUNC(pow,   "x, y");
  DECLARE_FUNC(sqrt,  "x");
  DECLARE_FUNC(ceil,  "x");
  DECLARE_FUNC(fabs,  "x");
  DECLARE_FUNC(floor, "x");
  DECLARE_FUNC(fmod,  "x, y");
  DECLARE_FUNC(round, "x, y");

#define DECLARE_FLOAT(name, value) \
  getEnvironment()->declareFloatConstant(#name, value)
#define DECLARE_INT(name, value) \
  getEnvironment()->declareIntegerConstant(#name, value)

  DECLARE_FLOAT(PI, M_PI);

  DECLARE_FLOAT(FLOAT_EPSILON, numeric_limits<double>::epsilon());
  DECLARE_FLOAT(FLOAT_MAX,     numeric_limits<double>::max());
  DECLARE_FLOAT(FLOAT_MIN,     numeric_limits<double>::min());
  DECLARE_FLOAT(FLOAT_MAX_EXP, numeric_limits<double>::max_exponent);
  DECLARE_FLOAT(FLOAT_MIN_EXP, numeric_limits<double>::min_exponent);
  DECLARE_INT(INTEGER_MAX,     numeric_limits<uint32_t>::max());
  DECLARE_INT(INTEGER_MIN,     numeric_limits<uint32_t>::min());
}


SmartPointer<Value> MathLibrary::near(Context &ctx, const Arguments &args) {
  double a = args[0]->toFloat();
  double b = args[0]->toFloat();
  double delta = args[1]->toFloat();

  return new BoolValue(a - delta <= b && b <= a + delta);
}


SmartPointer<Value> MathLibrary::min(Context &ctx, const Arguments &args) {
  double a = args[0]->toFloat();
  double b = args[0]->toFloat();

  return new FloatValue(a < b ? a : b);
}


SmartPointer<Value> MathLibrary::max(Context &ctx, const Arguments &args) {
  double a = args[0]->toFloat();
  double b = args[0]->toFloat();

  return new FloatValue(a < b ? b : a);
}


#ifdef _WIN32
double round(double x) {return floor(x + 0.5);}
#endif

#define MATH_FUNC1(name)                                                \
  SmartPointer<Value> MathLibrary::name(Context &ctx, const Arguments &args) { \
    return new FloatValue(::name(args[0]->toFloat()));                \
  }

#define MATH_FUNC2(name)                                                \
  SmartPointer<Value> MathLibrary::name(Context &ctx, const Arguments &args) { \
    return new FloatValue(::name(args[0]->toFloat(), args[1]->toFloat())); \
  }

MATH_FUNC1(acos)
MATH_FUNC1(asin)
MATH_FUNC1(atan)
MATH_FUNC2(atan2)
MATH_FUNC1(cos)
MATH_FUNC1(cosh)
MATH_FUNC1(sin)
MATH_FUNC1(sinh)
MATH_FUNC1(tan)
MATH_FUNC1(tanh)
MATH_FUNC1(exp)
MATH_FUNC1(log)
MATH_FUNC1(log10)
MATH_FUNC2(pow)
MATH_FUNC1(sqrt)
MATH_FUNC1(ceil)
MATH_FUNC1(fabs)
MATH_FUNC1(floor)
MATH_FUNC2(fmod)
MATH_FUNC1(round)
