#ifndef OPENSCAM_TPL_VALUE_H
#define OPENSCAM_TPL_VALUE_H

#include "../TPLID.h"

#include <cbang/Exception.h>
#include <cbang/SmartPointer.h>
#include <cbang/StdTypes.h>

#include <vector>


namespace OpenSCAM {
  namespace TPL {
    class Value : public TPLID {
    public:
      virtual ~Value() {}

      bool operator==(const Value &value) const;

      bool isNull() const {return getType() == TPLID_NULL;}
      bool isBool() const {return getType() == TPLID_BOOL;}
      bool isInteger() const {return getType() == TPLID_INTEGER;}
      bool isFloat() const {return getType() == TPLID_FLOAT;}
      bool isNumber() const {return isInteger() || isFloat();}
      bool isString() const {return getType() == TPLID_STRING;}
      bool isVector() const {return getType() == TPLID_VECTOR;}

      void appendNull();
      void appendBool(bool value);
      void appendInteger(int32_t value);
      void appendFloat(double value);
      void appendString(const std::string &value);

      virtual TPLID getType() const = 0;
      virtual void append(const cb::SmartPointer<Value> &value)
      {CBANG_THROWS("Cannot append value to " << getType());}
      virtual bool toBool() const = 0;
      virtual double toFloat() const
      {CBANG_THROWS("Cannot convert " << getType() << " to float");}
      virtual int32_t toInteger() const
      {CBANG_THROWS("Cannot convert " << getType() << " to integer");}
      virtual const std::string &toString() const
      {CBANG_THROWS("Cannot convert " << getType() << " to string");}
      virtual const std::vector<cb::SmartPointer<Value> > &toVector() const
      {CBANG_THROWS("Cannot convert " << getType() << " to vector");}
      virtual bool equal(const Value &value) const = 0;
      virtual bool less(const Value &value) const {
        CBANG_THROWS("Cannot compare " << getType() << " to "
                     << value.getType());
      }
    };
  }
}

#endif // OPENSCAM_TPL_VALUE_H

