/******************************************************************************\

    OpenSCAM is an Open-Source CAM software.
    Copyright (C) 2011-2012 Joseph Coffland <joseph@cauldrondevelopment.com>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

\******************************************************************************/

#ifndef OPENSCAM_FORMATTED_OBSERVER_H
#define OPENSCAM_FORMATTED_OBSERVER_H

#include "Observer.h"
#include "ValueFormatter.h"

#include <cbang/SmartPointer.h>


namespace OpenSCAM {
  class FormattedObserver : public Observer {
  protected:
    cb::SmartPointer<ValueFormatter> formatter;

  public:
    FormattedObserver(const cb::SmartPointer<ValueFormatter> &formatter) :
      formatter(formatter) {}

    virtual void formattedUpdate(const std::string &name,
                                 const std::string &value) = 0;

    // From Observer
    using Observer::updated;
    void updated(const std::string &name, const std::string &value)
    {formattedUpdate(name, formatter->format(value));}
    void updated(const std::string &name, int64_t value)
    {formattedUpdate(name, formatter->format(value));}
    void updated(const std::string &name, double value)
    {formattedUpdate(name, formatter->format(value));}
    void updated(const std::string &name, bool value)
    {formattedUpdate(name, formatter->format(value));}
  };
}

#endif // OPENSCAM_FORMATTED_OBSERVER_H

