/******************************************************************************\

    OpenSCAM is an Open-Source CAM software.
    Copyright (C) 2011-2012 Joseph Coffland <joseph@cauldrondevelopment.com>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

\******************************************************************************/

#include "StringFormatter.h"

#include <openscam/Real.h>

#include <cbang/Exception.h>
#include <cbang/time/HumanTime.h>
#include <cbang/util/HumanNumber.h>

#include <stdarg.h>

using namespace std;
using namespace cb;
using namespace OpenSCAM;


string StringFormatter::doFormat(const char *fmt, ...) {
  string result;

  va_list ap;
  va_start(ap, fmt);

  for (const char *ptr = fmt; *ptr;) {
    const char *start = ptr;

    if (*ptr == '%') {
      ptr++;

      // Flags
      bool flagsDone = false;
      while (!flagsDone) {
        switch (*ptr++) {
        case '#': case '0': case '-': case ' ': case '+': break;
        default:
          ptr--;
          flagsDone = true;
          break;
        }
      }

      // Width
      while ('0' <= *ptr && *ptr <= '9') ptr++;

      // Precision
      if (*ptr == '.') while ('0' <= *++ptr && *ptr <= '9') continue;

      // Length modifiers
      switch (*ptr++) {
      case 'h': if (*ptr == 'h') ptr++; break;
      case 'l': if (*ptr == 'l') ptr++; break;
      case 'L': case 'q': case 'j': case 'z': case 't': break;
      default: ptr--; break;
      }

      // Field conversion
      switch (*ptr++) {
      case 'i': case 'd': case 'o': case 'u': case 'x': case 'X':
      case 'e': case 'E': case 'f': case 'F': case 'g': case 'G':
      case 'a': case 'A': case 'c': case 's': case 'p': case '%':
        result += String::vprintf(string(start, ptr - start).c_str(), ap);
        break;

      case 'T':
        switch (*ptr++) {
        case 'f': result += HumanTime(va_arg(ap, double)); break;
        case 'i': result += HumanTime(va_arg(ap, int64_t)); break;
        default: THROW("Invalid HumanTime conversion");
        }
        break;

      case 'N':
        switch (*ptr++) {
        case 'f': result += HumanNumber(va_arg(ap, double)); break;
        case 'i': result += HumanNumber(va_arg(ap, int64_t)); break;
        default: THROW("Invalid HumanNumber conversion");
        }
        break;
      }

    } else result.append(1, *ptr++);
  }

  va_end(ap);

  return result;
}
