/******************************************************************************\

    OpenSCAM is an Open-Source CAM software.
    Copyright (C) 2011-2012 Joseph Coffland <joseph@cauldrondevelopment.com>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

\******************************************************************************/

#include <cbang/Exception.h>
#include <cbang/ApplicationMain.h>

#include <openscam/Application.h>

#include <openscam/tpl/Context.h>
#include <openscam/tpl/Environment.h>
#include <openscam/tpl/Interpreter.h>

#include <openscam/sim/Machine.h>
#include <openscam/sim/Controller.h>
#include <openscam/sim/ToolTable.h>
#include <openscam/cutsim/ToolPath.h>

#include <iostream>

using namespace std;
using namespace cb;
using namespace OpenSCAM;
using namespace OpenSCAM::TPL;


namespace OpenSCAM {
  class TPLInterpretApp : public Application, public Machine {
    Controller controller;
    ToolTable tools;
    ToolPath path;

  public:
    TPLInterpretApp() :
      Application("OpenSCAM TPL Interpreter"), controller(options, *this),
      path(tools) {}

    // From cb::Reader
    void run() {
      controller.init();
      Application::run();
      path.print(cout);
    }

    void read(const cb::InputSource &source) {
      Context ctx(controller);
      Interpreter(ctx).interpret(source);
    }

    // From Machine
    void move(const Move &move) {
      Machine::move(move);
      path.add(move);
    }
  };
}


int main(int argc, char *argv[]) {
  return cb::doApplication<OpenSCAM::TPLInterpretApp>(argc, argv);
}
